@@ -22,13 +22,12 @@
  * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
-#include <va/va_backend.h>
+#include "va_backend.h"
 
 #include "dummy_drv_video.h"
 
 #include "assert.h"
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
 
