@@ -27,11 +27,11 @@
  *
  */
 
-#include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 
-#include "va/x11/va_dricommon.h"
+#include "va_backend.h"
+#include "va_dricommon.h"
 
 #include "intel_driver.h"
 #include "intel_memman.h"
@@ -54,33 +54,6 @@ enum {
     I965_SURFACETYPE_INDEXED
 };
 
-/* List of supported image formats */
-typedef struct {
-    unsigned int        type;
-    VAImageFormat       va_format;
-} i965_image_format_map_t;
-
-static const i965_image_format_map_t
-i965_image_formats_map[I965_MAX_IMAGE_FORMATS + 1] = {
-    { I965_SURFACETYPE_YUV,
-      { VA_FOURCC('Y','V','1','2'), VA_LSB_FIRST, 12, } },
-    { I965_SURFACETYPE_YUV,
-      { VA_FOURCC('I','4','2','0'), VA_LSB_FIRST, 12, } },
-    { I965_SURFACETYPE_YUV,
-      { VA_FOURCC('N','V','1','2'), VA_LSB_FIRST, 12, } },
-};
-
-static const i965_image_format_map_t *get_image_format(unsigned int fourcc)
-{
-    unsigned int i;
-    for (i = 0; i965_image_formats_map[i].type != 0; i++) {
-        const i965_image_format_map_t * const m = &i965_image_formats_map[i];
-        if (m->va_format.fourcc == fourcc)
-            return m;
-    }
-    return NULL;
-}
-
 /* List of supported subpicture formats */
 typedef struct {
     unsigned int        type;
@@ -97,21 +70,13 @@ i965_subpic_formats_map[I965_MAX_SUBPIC_FORMATS + 1] = {
     { I965_SURFACETYPE_INDEXED, I965_SURFACEFORMAT_A4P4_UNORM,
       { VA_FOURCC('A','I','4','4'), VA_MSB_FIRST, 8, },
       0 },
-    { I965_SURFACETYPE_RGBA, I965_SURFACEFORMAT_B8G8R8A8_UNORM,
-      { VA_FOURCC('B','G','R','A'), VA_LSB_FIRST, 32,
-        32, 0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000 },
-      0 },
-    { I965_SURFACETYPE_RGBA, I965_SURFACEFORMAT_R8G8B8A8_UNORM,
-      { VA_FOURCC('R','G','B','A'), VA_LSB_FIRST, 32,
-        32, 0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000 },
-      0 },
 };
 
 static const i965_subpic_format_map_t *
 get_subpic_format(const VAImageFormat *va_format)
 {
     unsigned int i;
-    for (i = 0; i965_subpic_formats_map[i].type != 0; i++) {
+    for (i = 0; i < sizeof(i965_subpic_formats_map)/sizeof(i965_subpic_formats_map[0]); i++) {
         const i965_subpic_format_map_t * const m = &i965_subpic_formats_map[i];
         if (m->va_format.fourcc == va_format->fourcc &&
             (m->type == I965_SURFACETYPE_RGBA ?
@@ -134,11 +99,6 @@ i965_QueryConfigProfiles(VADriverContextP ctx,
 
     profile_list[i++] = VAProfileMPEG2Simple;
     profile_list[i++] = VAProfileMPEG2Main;
-#ifndef DISABLE_H264_DECODER
-    profile_list[i++] = VAProfileH264Baseline;
-    profile_list[i++] = VAProfileH264Main;
-    profile_list[i++] = VAProfileH264High;
-#endif
 
     /* If the assert fails then I965_MAX_PROFILES needs to be bigger */
     assert(i <= I965_MAX_PROFILES);
@@ -162,15 +122,6 @@ i965_QueryConfigEntrypoints(VADriverContextP ctx,
         entrypoint_list[0] = VAEntrypointVLD;
         break;
 
-#ifndef DISABLE_H264_DECODER
-    case VAProfileH264Baseline:
-    case VAProfileH264Main:
-    case VAProfileH264High:
-        *num_entrypoints = 1;
-        entrypoint_list[0] = VAEntrypointVLD;
-        break;
-#endif
-
     default:
         vaStatus = VA_STATUS_ERROR_UNSUPPORTED_PROFILE;
         *num_entrypoints = 0;
@@ -266,18 +217,6 @@ i965_CreateConfig(VADriverContextP ctx,
         }
         break;
 
-#ifndef DISABLE_H264_DECODER
-    case VAProfileH264Baseline:
-    case VAProfileH264Main:
-    case VAProfileH264High:
-        if (VAEntrypointVLD == entrypoint) {
-            vaStatus = VA_STATUS_SUCCESS;
-        } else {
-            vaStatus = VA_STATUS_ERROR_UNSUPPORTED_ENTRYPOINT;
-        }
-        break;
-#endif
-
     default:
         vaStatus = VA_STATUS_ERROR_UNSUPPORTED_PROFILE;
         break;
@@ -359,34 +298,13 @@ VAStatus i965_QueryConfigAttributes(VADriverContextP ctx,
     return vaStatus;
 }
 
-static struct object_image *
-i965_CreateImage_impl(
-    VADriverContextP     ctx,
-    const VAImageFormat *format,
-    unsigned int         width,
-    unsigned int         height,
-    VASurfaceID          surface
-);
-
-static void
-i965_DestroyImage_impl(
-    VADriverContextP     ctx,
-    struct object_image *obj_image,
-    int                  from_surface
-);
-
 static void 
 i965_destroy_surface(struct object_heap *heap, struct object_base *obj)
 {
     struct object_surface *obj_surface = (struct object_surface *)obj;
 
+    dri_bo_unreference(obj_surface->bo);
     obj_surface->bo = NULL;
-
-    if (obj_surface->free_private_data != NULL) {
-        obj_surface->free_private_data(&obj_surface->private_data);
-        obj_surface->private_data = NULL;
-    }
-
     object_heap_free(heap, obj);
 }
 
@@ -410,8 +328,6 @@ i965_CreateSurfaces(VADriverContextP ctx,
     for (i = 0; i < num_surfaces; i++) {
         int surfaceID = NEW_SURFACE_ID();
         struct object_surface *obj_surface = SURFACE(surfaceID);
-        struct object_image *obj_image;
-        struct object_buffer *obj_buffer;
 
         if (NULL == obj_surface) {
             vaStatus = VA_STATUS_ERROR_ALLOCATION_FAILED;
@@ -421,27 +337,19 @@ i965_CreateSurfaces(VADriverContextP ctx,
         surfaces[i] = surfaceID;
         obj_surface->status = VASurfaceReady;
         obj_surface->subpic = VA_INVALID_ID;
-        obj_surface->width = ALIGN(width, 16);
-        obj_surface->height = ALIGN(height, 16);
-        obj_surface->is_derived = 0;
-        obj_surface->private_data = NULL;
-        obj_surface->free_private_data = NULL;
+        obj_surface->width = width;
+        obj_surface->height = height;
+        obj_surface->size = SIZE_YUV420(width, height);
+        obj_surface->bo = dri_bo_alloc(i965->intel.bufmgr,
+                                       "vaapi surface",
+                                       obj_surface->size,
+                                       64);
 
-        obj_image = i965_CreateImage_impl(ctx, NULL, obj_surface->width, obj_surface->height, surfaceID);
-        if (!obj_image) {
-            vaStatus = VA_STATUS_ERROR_ALLOCATION_FAILED;
+        assert(obj_surface->bo);
+        if (NULL == obj_surface->bo) {
+            vaStatus = VA_STATUS_ERROR_UNKNOWN;
             break;
         }
-        obj_surface->size = obj_image->image.data_size;
-        obj_surface->image = obj_image->image;
-
-        obj_buffer = BUFFER(obj_image->image.buf);
-        if (!obj_buffer) {
-            vaStatus = VA_STATUS_ERROR_ALLOCATION_FAILED;
-            break;
-        }
-        assert(obj_buffer->buffer_store);
-        obj_surface->bo = obj_buffer->buffer_store->bo;
     }
 
     /* Error recovery */
@@ -471,7 +379,6 @@ i965_DestroySurfaces(VADriverContextP ctx,
         struct object_surface *obj_surface = SURFACE(surface_list[i]);
 
         assert(obj_surface);
-        i965_DestroyImage_impl(ctx, IMAGE(obj_surface->image.image_id), 1);
         i965_destroy_surface(&i965->surface_heap, (struct object_base *)obj_surface);
     }
 
@@ -483,20 +390,28 @@ i965_QueryImageFormats(VADriverContextP ctx,
                        VAImageFormat *format_list,      /* out */
                        int *num_formats)                /* out */
 {
-    int n;
-
-    for (n = 0; i965_image_formats_map[n].va_format.fourcc != 0; n++) {
-        const i965_image_format_map_t * const m = &i965_image_formats_map[n];
-        if (format_list)
-            format_list[n] = m->va_format;
-    }
-
     if (num_formats)
-        *num_formats = n;
+        *num_formats = 0;
 
     return VA_STATUS_SUCCESS;
 }
 
+VAStatus 
+i965_PutImage(VADriverContextP ctx,
+               VASurfaceID surface,
+               VAImageID image,
+               int src_x,
+               int src_y,
+               unsigned int src_width,
+               unsigned int src_height,
+               int dest_x,
+               int dest_y,
+               unsigned int dest_width,
+               unsigned int dest_height)
+{
+    return VA_STATUS_SUCCESS;
+}
+
 VAStatus 
 i965_QuerySubpictureFormats(VADriverContextP ctx,
                             VAImageFormat *format_list,         /* out */
@@ -552,7 +467,6 @@ i965_CreateSubpicture(VADriverContextP ctx,
     obj_subpic->format = m->format;
     obj_subpic->width  = obj_image->image.width;
     obj_subpic->height = obj_image->image.height;
-    obj_subpic->pitch  = obj_image->image.pitches[0];
     obj_subpic->bo     = obj_image->bo;
     return VA_STATUS_SUCCESS;
 }
@@ -573,8 +487,7 @@ i965_SetSubpictureImage(VADriverContextP ctx,
                         VASubpictureID subpicture,
                         VAImageID image)
 {
-    /* TODO */
-    return VA_STATUS_ERROR_UNIMPLEMENTED;
+    return VA_STATUS_SUCCESS;
 }
 
 VAStatus 
@@ -584,8 +497,7 @@ i965_SetSubpictureChromakey(VADriverContextP ctx,
                             unsigned int chromakey_max,
                             unsigned int chromakey_mask)
 {
-    /* TODO */
-    return VA_STATUS_ERROR_UNIMPLEMENTED;
+    return VA_STATUS_SUCCESS;
 }
 
 VAStatus 
@@ -593,8 +505,7 @@ i965_SetSubpictureGlobalAlpha(VADriverContextP ctx,
                               VASubpictureID subpicture,
                               float global_alpha)
 {
-    /* TODO */
-    return VA_STATUS_ERROR_UNIMPLEMENTED;
+    return VA_STATUS_SUCCESS;
 }
 
 VAStatus 
@@ -697,23 +608,12 @@ static void
 i965_destroy_context(struct object_heap *heap, struct object_base *obj)
 {
     struct object_context *obj_context = (struct object_context *)obj;
-    int i;
-
-    assert(obj_context->decode_state.num_slice_params <= obj_context->decode_state.max_slice_params);
-    assert(obj_context->decode_state.num_slice_datas <= obj_context->decode_state.max_slice_datas);
 
     i965_release_buffer_store(&obj_context->decode_state.pic_param);
+    i965_release_buffer_store(&obj_context->decode_state.slice_param);
     i965_release_buffer_store(&obj_context->decode_state.iq_matrix);
     i965_release_buffer_store(&obj_context->decode_state.bit_plane);
-
-    for (i = 0; i < obj_context->decode_state.num_slice_params; i++)
-        i965_release_buffer_store(&obj_context->decode_state.slice_params[i]);
-
-    for (i = 0; i < obj_context->decode_state.num_slice_datas; i++)
-        i965_release_buffer_store(&obj_context->decode_state.slice_datas[i]);
-
-    free(obj_context->decode_state.slice_params);
-    free(obj_context->decode_state.slice_datas);
+    i965_release_buffer_store(&obj_context->decode_state.slice_data);
     free(obj_context->render_targets);
     object_heap_free(heap, obj);
 }
@@ -754,12 +654,6 @@ i965_CreateContext(VADriverContextP ctx,
     *context = contextID;
     memset(&obj_context->decode_state, 0, sizeof(obj_context->decode_state));
     obj_context->decode_state.current_render_target = -1;
-    obj_context->decode_state.max_slice_params = NUM_SLICES;
-    obj_context->decode_state.max_slice_datas = NUM_SLICES;
-    obj_context->decode_state.slice_params = calloc(obj_context->decode_state.max_slice_params,
-                                                    sizeof(*obj_context->decode_state.slice_params));
-    obj_context->decode_state.slice_datas = calloc(obj_context->decode_state.max_slice_datas,
-                                                   sizeof(*obj_context->decode_state.slice_datas));
     obj_context->config_id = config_id;
     obj_context->picture_width = picture_width;
     obj_context->picture_height = picture_height;
@@ -873,7 +767,6 @@ i965_CreateBuffer(VADriverContextP ctx,
             memcpy(buffer_store->buffer, data, size * num_elements);
     }
 
-    buffer_store->num_elements = obj_buffer->num_elements;
     i965_reference_buffer_store(&obj_buffer->buffer_store, buffer_store);
     i965_release_buffer_store(&buffer_store);
     *buf_id = bufferID;
@@ -988,14 +881,6 @@ i965_BeginPicture(VADriverContextP ctx,
         vaStatus = VA_STATUS_SUCCESS;
         break;
 
-#ifndef DISABLE_H264_DECODER
-    case VAProfileH264Baseline:
-    case VAProfileH264Main:
-    case VAProfileH264High:
-        vaStatus = VA_STATUS_SUCCESS;
-        break;
-#endif
-
     default:
         assert(0);
         vaStatus = VA_STATUS_ERROR_UNSUPPORTED_PROFILE;
@@ -1056,18 +941,10 @@ i965_render_slice_parameter_buffer(VADriverContextP ctx,
 {
     assert(obj_buffer->buffer_store->bo == NULL);
     assert(obj_buffer->buffer_store->buffer);
-    
-    if (obj_context->decode_state.num_slice_params == obj_context->decode_state.max_slice_params) {
-        obj_context->decode_state.slice_params = realloc(obj_context->decode_state.slice_params,
-                                                        (obj_context->decode_state.max_slice_params + NUM_SLICES) * sizeof(*obj_context->decode_state.slice_params));
-        memset(obj_context->decode_state.slice_params + obj_context->decode_state.max_slice_params, 0, NUM_SLICES * sizeof(*obj_context->decode_state.slice_params));
-        obj_context->decode_state.max_slice_params += NUM_SLICES;
-    }
-        
-    i965_release_buffer_store(&obj_context->decode_state.slice_params[obj_context->decode_state.num_slice_params]);
-    i965_reference_buffer_store(&obj_context->decode_state.slice_params[obj_context->decode_state.num_slice_params],
+    i965_release_buffer_store(&obj_context->decode_state.slice_param);
+    i965_reference_buffer_store(&obj_context->decode_state.slice_param,
                                 obj_buffer->buffer_store);
-    obj_context->decode_state.num_slice_params++;
+    obj_context->decode_state.num_slices = obj_buffer->num_elements;    
     
     return VA_STATUS_SUCCESS;
 }
@@ -1079,18 +956,9 @@ i965_render_slice_data_buffer(VADriverContextP ctx,
 {
     assert(obj_buffer->buffer_store->buffer == NULL);
     assert(obj_buffer->buffer_store->bo);
-
-    if (obj_context->decode_state.num_slice_datas == obj_context->decode_state.max_slice_datas) {
-        obj_context->decode_state.slice_datas = realloc(obj_context->decode_state.slice_datas,
-                                                        (obj_context->decode_state.max_slice_datas + NUM_SLICES) * sizeof(*obj_context->decode_state.slice_datas));
-        memset(obj_context->decode_state.slice_datas + obj_context->decode_state.max_slice_datas, 0, NUM_SLICES * sizeof(*obj_context->decode_state.slice_datas));
-        obj_context->decode_state.max_slice_datas += NUM_SLICES;
-    }
-        
-    i965_release_buffer_store(&obj_context->decode_state.slice_datas[obj_context->decode_state.num_slice_datas]);
-    i965_reference_buffer_store(&obj_context->decode_state.slice_datas[obj_context->decode_state.num_slice_datas],
+    i965_release_buffer_store(&obj_context->decode_state.slice_data);
+    i965_reference_buffer_store(&obj_context->decode_state.slice_data,
                                 obj_buffer->buffer_store);
-    obj_context->decode_state.num_slice_datas++;
     
     return VA_STATUS_SUCCESS;
 }
@@ -1146,46 +1014,26 @@ VAStatus
 i965_EndPicture(VADriverContextP ctx, VAContextID context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx); 
-    struct i965_render_state *render_state = &i965->render_state;
     struct object_context *obj_context = CONTEXT(context);
     struct object_config *obj_config;
     VAContextID config;
-    int i;
 
     assert(obj_context);
     assert(obj_context->decode_state.pic_param);
-    assert(obj_context->decode_state.num_slice_params >= 1);
-    assert(obj_context->decode_state.num_slice_datas >= 1);
-    assert(obj_context->decode_state.num_slice_params == obj_context->decode_state.num_slice_datas);
+    assert(obj_context->decode_state.slice_param);
+    assert(obj_context->decode_state.slice_data);
 
     config = obj_context->config_id;
     obj_config = CONFIG(config);
     assert(obj_config);
-
-    switch (obj_config->profile) {
-#ifndef DISABLE_H264_DECODER
-    case VAProfileH264Baseline:
-    case VAProfileH264Main:
-    case VAProfileH264High:
-        render_state->interleaved_uv = 1;
-        break;
-#endif
-    default:
-        render_state->interleaved_uv = 0;
-    }
-
     i965_media_decode_picture(ctx, obj_config->profile, &obj_context->decode_state);
     obj_context->decode_state.current_render_target = -1;
-    obj_context->decode_state.num_slice_params = 0;
-    obj_context->decode_state.num_slice_datas = 0;
+    obj_context->decode_state.num_slices = 0;
     i965_release_buffer_store(&obj_context->decode_state.pic_param);
+    i965_release_buffer_store(&obj_context->decode_state.slice_param);
     i965_release_buffer_store(&obj_context->decode_state.iq_matrix);
     i965_release_buffer_store(&obj_context->decode_state.bit_plane);
-
-    for (i = 0; i < obj_context->decode_state.num_slice_params; i++) {
-        i965_release_buffer_store(&obj_context->decode_state.slice_params[i]);
-        i965_release_buffer_store(&obj_context->decode_state.slice_datas[i]);
-    }
+    i965_release_buffer_store(&obj_context->decode_state.slice_data);
 
     return VA_STATUS_SUCCESS;
 }
@@ -1246,7 +1094,7 @@ i965_GetDisplayAttributes(VADriverContextP ctx,
                           int num_attributes)
 {
     /* TODO */
-    return VA_STATUS_ERROR_UNIMPLEMENTED;
+    return VA_STATUS_ERROR_UNKNOWN;
 }
 
 /* 
@@ -1261,7 +1109,7 @@ i965_SetDisplayAttributes(VADriverContextP ctx,
                           int num_attributes)
 {
     /* TODO */
-    return VA_STATUS_ERROR_UNIMPLEMENTED;
+    return VA_STATUS_ERROR_UNKNOWN;
 }
 
 VAStatus 
@@ -1271,7 +1119,7 @@ i965_DbgCopySurfaceToBuffer(VADriverContextP ctx,
                             unsigned int *stride)       /* out */
 {
     /* TODO */
-    return VA_STATUS_ERROR_UNIMPLEMENTED;
+    return VA_STATUS_ERROR_UNKNOWN;
 }
 
 static VAStatus 
@@ -1283,7 +1131,7 @@ i965_Init(VADriverContextP ctx)
         return VA_STATUS_ERROR_UNKNOWN;
 
     if (!IS_G4X(i965->intel.device_id) &&
-        !IS_IRONLAKE(i965->intel.device_id))
+        !IS_IGDNG(i965->intel.device_id))
         return VA_STATUS_ERROR_UNKNOWN;
 
     if (i965_media_init(ctx) == False)
@@ -1318,22 +1166,46 @@ i965_destroy_heap(struct object_heap *heap,
 VAStatus 
 i965_DestroyImage(VADriverContextP ctx, VAImageID image);
 
-static int
-setup_image_format(
-    VAImage     *image,
-    unsigned int fourcc,
-    unsigned int width,
-    unsigned int height
-)
+VAStatus 
+i965_CreateImage(VADriverContextP ctx,
+                 VAImageFormat *format,
+                 int width,
+                 int height,
+                 VAImage *out_image)        /* out */
 {
+    struct i965_driver_data *i965 = i965_driver_data(ctx);
+    struct object_image *obj_image;
+    VAStatus va_status = VA_STATUS_ERROR_OPERATION_FAILED;
+    VAImageID image_id;
     unsigned int width2, height2, size2, size;
 
+    out_image->image_id = VA_INVALID_ID;
+    out_image->buf      = VA_INVALID_ID;
+
+    image_id = NEW_IMAGE_ID();
+    if (image_id == VA_INVALID_ID)
+        return VA_STATUS_ERROR_ALLOCATION_FAILED;
+
+    obj_image = IMAGE(image_id);
+    if (!obj_image)
+        return VA_STATUS_ERROR_ALLOCATION_FAILED;
+    obj_image->bo         = NULL;
+    obj_image->palette    = NULL;
+
+    VAImage * const image = &obj_image->image;
+    image->image_id       = image_id;
+    image->buf            = VA_INVALID_ID;
+
     size    = width * height;
     width2  = (width  + 1) / 2;
     height2 = (height + 1) / 2;
     size2   = width2 * height2;
 
-    switch (fourcc) {
+    image->num_palette_entries = 0;
+    image->entry_bytes         = 0;
+    memset(image->component_order, 0, sizeof(image->component_order));
+
+    switch (format->fourcc) {
     case VA_FOURCC('I','A','4','4'):
     case VA_FOURCC('A','I','4','4'):
         image->num_planes = 1;
@@ -1346,94 +1218,16 @@ setup_image_format(
         image->component_order[1]  = 'G';
         image->component_order[2]  = 'B';
         break;
-    case VA_FOURCC('A','R','G','B'):
-    case VA_FOURCC('A','B','G','R'):
-    case VA_FOURCC('B','G','R','A'):
-    case VA_FOURCC('R','G','B','A'):
-        image->num_planes = 1;
-        image->pitches[0] = width * 4;
-        image->offsets[0] = 0;
-        image->data_size  = image->offsets[0] + image->pitches[0] * height;
-        break;
-    case VA_FOURCC('Y','V','1','2'):
-        image->num_planes = 3;
-        image->pitches[0] = width;
-        image->offsets[0] = 0;
-        image->pitches[1] = width2;
-        image->offsets[1] = size + size2;
-        image->pitches[2] = width2;
-        image->offsets[2] = size;
-        image->data_size  = size + 2 * size2;
-        break;
-    case VA_FOURCC('I','4','2','0'):
-        image->num_planes = 3;
-        image->pitches[0] = width;
-        image->offsets[0] = 0;
-        image->pitches[1] = width2;
-        image->offsets[1] = size;
-        image->pitches[2] = width2;
-        image->offsets[2] = size + size2;
-        image->data_size  = size + 2 * size2;
-        break;
-    case VA_FOURCC('N','V','1','2'):
-        image->num_planes = 2;
-        image->pitches[0] = width;
-        image->offsets[0] = 0;
-        image->pitches[1] = width;
-        image->offsets[1] = size;
-        image->data_size  = size + 2 * size2;
-        break;
     default:
-        return 0;
+        goto error;
     }
-    return 1;
-}
 
-static struct object_image *
-i965_CreateImage_impl(
-    VADriverContextP     ctx,
-    const VAImageFormat *format,
-    unsigned int         width,
-    unsigned int         height,
-    VASurfaceID          surface
-)
-{
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct object_image *obj_image;
-    VAImageID image_id;
+    va_status = i965_CreateBuffer(ctx, 0, VAImageBufferType,
+                                  image->data_size, 1, NULL, &image->buf);
+    if (va_status != VA_STATUS_SUCCESS)
+        goto error;
 
-    image_id = NEW_IMAGE_ID();
-    if (image_id == VA_INVALID_ID)
-        return NULL;
-
-    obj_image = IMAGE(image_id);
-    if (!obj_image)
-        return NULL;
-    obj_image->surface    = surface;
-    obj_image->bo         = NULL;
-    obj_image->palette    = NULL;
-
-    VAImage * const image = &obj_image->image;
-    image->image_id       = image_id;
-    image->buf            = VA_INVALID_ID;
-
-    image->num_palette_entries = 0;
-    image->entry_bytes         = 0;
-    memset(image->component_order, 0, sizeof(image->component_order));
-
-    if (surface == VA_INVALID_ID) {
-        if (!setup_image_format(image, format->fourcc, width, height))
-            goto error;
-        image->format = *format;
-    }
-    else {
-        /* VA image format will be initialized through first vaDeriveImage() */
-        if (!setup_image_format(image, VA_FOURCC_YV12, width, height))
-            goto error;
-        memset(&image->format, 0, sizeof(image->format));
-    }
-    image->width  = width;
-    image->height = height;
+    obj_image->bo = BUFFER(image->buf)->buffer_store->bo;
 
     if (image->num_palette_entries > 0 && image->entry_bytes > 0) {
         obj_image->palette = malloc(image->num_palette_entries * sizeof(obj_image->palette));
@@ -1441,65 +1235,23 @@ i965_CreateImage_impl(
             goto error;
     }
 
-    if (i965_CreateBuffer(ctx, 0, VAImageBufferType,
-                          image->data_size, 1, NULL,
-                          &image->buf) != VA_STATUS_SUCCESS)
-        goto error;
+    image->image_id             = image_id;
+    image->format               = *format;
+    image->width                = width;
+    image->height               = height;
 
-    obj_image->bo = BUFFER(image->buf)->buffer_store->bo;
-    return obj_image;
+    *out_image                  = *image;
+    return VA_STATUS_SUCCESS;
 
  error:
-    i965_DestroyImage_impl(ctx, obj_image, 0);
-    return NULL;
-}
-
-VAStatus 
-i965_CreateImage(VADriverContextP ctx,
-                 VAImageFormat *format,
-                 int width,
-                 int height,
-                 VAImage *out_image)        /* out */
-{
-    struct object_image *obj_image;
-
-    obj_image = i965_CreateImage_impl(ctx, format, width, height, VA_INVALID_ID);
-    if (!obj_image)
-        return VA_STATUS_ERROR_ALLOCATION_FAILED;
-
-    *out_image = obj_image->image;
-    return VA_STATUS_SUCCESS;
+    i965_DestroyImage(ctx, image_id);
+    return va_status;
 }
 
 VAStatus i965_DeriveImage(VADriverContextP ctx,
                           VASurfaceID surface,
                           VAImage *image)        /* out */
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_render_state *render_state = &i965->render_state;
-
-    struct object_surface *obj_surface = SURFACE(surface);
-    if (!obj_surface)
-        return VA_STATUS_ERROR_INVALID_SURFACE;
-
-    struct object_image *obj_image = IMAGE(obj_surface->image.image_id);
-    if (!obj_image)
-        return VA_STATUS_ERROR_INVALID_IMAGE;
-
-    if (!obj_surface->is_derived) {
-        /* Reset VA image format on each call so that a VA surface can
-           be re-used with another context (and internal surface format) */
-        VAImage * const image = &obj_surface->image;
-        const i965_image_format_map_t *m;
-        m = get_image_format(render_state->interleaved_uv ? VA_FOURCC_NV12 : VA_FOURCC_YV12);
-        if (!m)
-            return VA_STATUS_ERROR_OPERATION_FAILED;
-        if (!setup_image_format(image, m->va_format.fourcc, image->width, image->height))
-            return VA_STATUS_ERROR_OPERATION_FAILED;
-        image->format = m->va_format;
-        obj_surface->is_derived = 1;
-    }
-    *image = obj_surface->image;
     return VA_STATUS_SUCCESS;
 }
 
@@ -1509,24 +1261,15 @@ i965_destroy_image(struct object_heap *heap, struct object_base *obj)
     object_heap_free(heap, obj);
 }
 
-static void
-i965_DestroyImage_impl(
-    VADriverContextP     ctx,
-    struct object_image *obj_image,
-    int                  from_surface
-)
+
+VAStatus 
+i965_DestroyImage(VADriverContextP ctx, VAImageID image)
 {
-    struct i965_driver_data * const i965 = i965_driver_data(ctx);
+    struct i965_driver_data *i965 = i965_driver_data(ctx);
+    struct object_image *obj_image = IMAGE(image); 
 
     if (!obj_image)
-        return;
-
-    if (!from_surface && obj_image->surface != VA_INVALID_ID) {
-        /* only destroy when called from vaDestroySurfaces() */
-        struct object_surface *obj_surface = SURFACE(obj_image->surface);
-        obj_surface->is_derived = 0;
-        return;
-    }
+        return VA_STATUS_SUCCESS;
 
     if (obj_image->image.buf != VA_INVALID_ID) {
         i965_DestroyBuffer(ctx, obj_image->image.buf);
@@ -1539,15 +1282,6 @@ i965_DestroyImage_impl(
     }
 
     i965_destroy_image(&i965->image_heap, (struct object_base *)obj_image);
-}
-
-VAStatus 
-i965_DestroyImage(VADriverContextP ctx, VAImageID image)
-{
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct object_image *obj_image = IMAGE(image); 
-
-    i965_DestroyImage_impl(ctx, obj_image, 0);
 	
     return VA_STATUS_SUCCESS;
 }
@@ -1588,142 +1322,7 @@ i965_GetImage(VADriverContextP ctx,
               unsigned int height,
               VAImageID image)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_render_state *render_state = &i965->render_state;
-
-    struct object_surface *obj_surface = SURFACE(surface);
-    if (!obj_surface)
-        return VA_STATUS_ERROR_INVALID_SURFACE;
-
-    if (obj_surface->is_derived)
-        return VA_STATUS_ERROR_SURFACE_BUSY;
-
-    struct object_image *obj_image = IMAGE(image);
-    if (!obj_image)
-        return VA_STATUS_ERROR_INVALID_IMAGE;
-
-    /* XXX: we only support full-size surface readback */
-    if (x != 0 ||
-        y != 0 ||
-        width != obj_surface->width ||
-        height != obj_surface->height)
-        return VA_STATUS_ERROR_INVALID_PARAMETER;
-
-    /* XXX: we only support 1:1 image copies */
-    if (width != obj_image->image.width ||
-        height != obj_image->image.height)
-        return VA_STATUS_ERROR_INVALID_PARAMETER;
-
-    VAStatus va_status;
-    void *image_data = NULL;
-
-    va_status = i965_MapBuffer(ctx, obj_image->image.buf, &image_data);
-    if (va_status != VA_STATUS_SUCCESS)
-        return va_status;
-
-    dri_bo_map(obj_surface->bo, 0);
-
-    switch (obj_image->image.format.fourcc) {
-    case VA_FOURCC('Y','V','1','2'):
-    case VA_FOURCC('I','4','2','0'):
-        /* I420 is native format for MPEG-2 decoded surfaces */
-        if (render_state->interleaved_uv)
-            goto operation_failed;
-        memcpy(image_data, obj_surface->bo->virtual, obj_surface->bo->size);
-        break;
-    case VA_FOURCC('N','V','1','2'):
-        /* NV12 is native format for H.264 decoded surfaces */
-        if (!render_state->interleaved_uv)
-            goto operation_failed;
-        memcpy(image_data, obj_surface->bo->virtual, obj_surface->bo->size);
-        break;
-    default:
-    operation_failed:
-        va_status = VA_STATUS_ERROR_OPERATION_FAILED;
-        break;
-    }
-
-    dri_bo_unmap(obj_surface->bo);
-
-    i965_UnmapBuffer(ctx, obj_image->image.buf);
-    return va_status;
-}
-
-VAStatus 
-i965_PutImage(VADriverContextP ctx,
-               VASurfaceID surface,
-               VAImageID image,
-               int src_x,
-               int src_y,
-               unsigned int src_width,
-               unsigned int src_height,
-               int dest_x,
-               int dest_y,
-               unsigned int dest_width,
-               unsigned int dest_height)
-{
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_render_state *render_state = &i965->render_state;
-
-    struct object_surface *obj_surface = SURFACE(surface);
-    if (!obj_surface)
-        return VA_STATUS_ERROR_INVALID_SURFACE;
-
-    if (obj_surface->is_derived)
-        return VA_STATUS_ERROR_SURFACE_BUSY;
-
-    struct object_image *obj_image = IMAGE(image);
-    if (!obj_image)
-        return VA_STATUS_ERROR_INVALID_IMAGE;
-
-    /* XXX: we don't support partial video surface updates */
-    if (src_x != 0 ||
-        src_y != 0 ||
-        src_width != obj_image->image.width ||
-        src_height != obj_image->image.height)
-        return VA_STATUS_ERROR_OPERATION_FAILED;
-    if (dest_x != 0 ||
-        dest_y != 0 ||
-        dest_width != obj_surface->width ||
-        dest_height != obj_surface->height)
-        return VA_STATUS_ERROR_OPERATION_FAILED;
-    if (src_width != dest_width ||
-        src_height != dest_height)
-        return VA_STATUS_ERROR_OPERATION_FAILED;
-
-    VAStatus va_status;
-    void *image_data = NULL;
-
-    va_status = i965_MapBuffer(ctx, obj_image->image.buf, &image_data);
-    if (va_status != VA_STATUS_SUCCESS)
-        return va_status;
-
-    dri_bo_map(obj_surface->bo, 1);
-
-    switch (obj_image->image.format.fourcc) {
-    case VA_FOURCC('Y','V','1','2'):
-    case VA_FOURCC('I','4','2','0'):
-        /* I420 is native format for MPEG-2 decoded surfaces */
-        if (render_state->interleaved_uv)
-            goto operation_failed;
-        memcpy(obj_surface->bo->virtual, image_data, obj_surface->bo->size);
-        break;
-    case VA_FOURCC('N','V','1','2'):
-        /* NV12 is native format for H.264 decoded surfaces */
-        if (!render_state->interleaved_uv)
-            goto operation_failed;
-        memcpy(obj_surface->bo->virtual, image_data, obj_surface->bo->size);
-        break;
-    default:
-    operation_failed:
-        va_status = VA_STATUS_ERROR_OPERATION_FAILED;
-        break;
-    }
-
-    dri_bo_unmap(obj_surface->bo);
-
-    i965_UnmapBuffer(ctx, obj_image->image.buf);
-    return va_status;
+    return VA_STATUS_SUCCESS;
 }
 
 VAStatus 
@@ -1749,23 +1348,13 @@ i965_PutSurface(VADriverContextP ctx,
     union dri_buffer *buffer;
     struct intel_region *dest_region;
     struct object_surface *obj_surface; 
-    int ret;
+	int ret;
     uint32_t name;
-    VARectangle src_rect, dst_rect;
     Bool new_region = False;
-
     /* Currently don't support DRI1 */
     if (dri_state->driConnectedFlag != VA_DRI2)
         return VA_STATUS_ERROR_UNKNOWN;
 
-    /* XXX: we currently only support up to 64 cliprects */
-    if (number_cliprects > MAX_CLIP_RECTS)
-        return VA_STATUS_ERROR_OPERATION_FAILED;
-
-    obj_surface = SURFACE(surface);
-    if (!obj_surface)
-        return VA_STATUS_ERROR_INVALID_SURFACE;
-
     dri_drawable = dri_get_drawable(ctx, draw);
     assert(dri_drawable);
 
@@ -1804,24 +1393,17 @@ i965_PutSurface(VADriverContextP ctx,
         assert(ret == 0);
     }
 
-    src_rect.x      = srcx;
-    src_rect.y      = srcy;
-    src_rect.width  = srcw;
-    src_rect.height = srch;
-
-    dst_rect.x      = destx;
-    dst_rect.y      = desty;
-    dst_rect.width  = destw;
-    dst_rect.height = desth;
-
-    i965_render_put_surface(ctx, surface, &src_rect, &dst_rect,
-                            cliprects, number_cliprects);
-
-    if (obj_surface->subpic != VA_INVALID_ID)
-	i965_render_put_subpic(ctx, surface, &src_rect, &dst_rect,
-                               cliprects, number_cliprects);
-
+    i965_render_put_surface(ctx, surface,
+                            srcx, srcy, srcw, srch,
+                            destx, desty, destw, desth);
+    obj_surface = SURFACE(surface);
+    if(obj_surface->subpic != VA_INVALID_ID) {	
+	i965_render_put_subpic(ctx, surface,
+                           srcx, srcy, srcw, srch,
+                           destx, desty, destw, desth);
+    } 
     dri_swap_buffer(ctx, dri_drawable);
+
     return VA_STATUS_SUCCESS;
 }
 
@@ -1852,10 +1434,6 @@ i965_Terminate(VADriverContextP ctx)
     return VA_STATUS_SUCCESS;
 }
 
-VAStatus 
-__vaDriverInit_0_31(  VADriverContextP ctx )
-    __attribute__((__visibility__("default")));
-
 VAStatus 
 __vaDriverInit_0_31(  VADriverContextP ctx )
 {
