@@ -31,7 +31,7 @@
 #include <string.h>
 #include <assert.h>
 
-#include <va/va_backend.h>
+#include "va_backend.h"
 
 #include "intel_batchbuffer.h"
 #include "intel_driver.h"
@@ -280,7 +280,7 @@ static struct media_kernel  mpeg2_vld_kernels_gen4[] = {
     }
 };
 
-/* On IRONLAKE */
+/* On IGDNG */
 static uint32_t frame_intra_kernel_gen5[][4] = {
    #include "shaders/mpeg2/vld/frame_intra.g4b.gen5"
 };
@@ -851,33 +851,30 @@ i965_media_mpeg2_states_setup(VADriverContextP ctx, struct decode_state *decode_
 static void
 i965_media_mpeg2_objects(VADriverContextP ctx, struct decode_state *decode_state)
 {
-    int i, j;
+    int i;
     VASliceParameterBufferMPEG2 *slice_param;
 
-    for (j = 0; j < decode_state->num_slice_params; j++) {
-        assert(decode_state->slice_params[j] && decode_state->slice_params[j]->buffer);
-        assert(decode_state->slice_datas[j] && decode_state->slice_datas[j]->bo);
-        slice_param = (VASliceParameterBufferMPEG2 *)decode_state->slice_params[j]->buffer;
+    assert(decode_state->slice_param && decode_state->slice_param->buffer);
+    slice_param = (VASliceParameterBufferMPEG2 *)decode_state->slice_param->buffer;
 
-        for (i = 0; i < decode_state->slice_params[j]->num_elements; i++) {
-            assert(slice_param->slice_data_flag == VA_SLICE_DATA_FLAG_ALL);
+    for (i = 0; i < decode_state->num_slices; i++) {
+        assert(slice_param->slice_data_flag == VA_SLICE_DATA_FLAG_ALL);
 
-            BEGIN_BATCH(ctx, 6);
-            OUT_BATCH(ctx, CMD_MEDIA_OBJECT | 4);
-            OUT_BATCH(ctx, 0);
-            OUT_BATCH(ctx, slice_param->slice_data_size - (slice_param->macroblock_offset >> 3));
-            OUT_RELOC(ctx, decode_state->slice_datas[j]->bo, 
-                      I915_GEM_DOMAIN_SAMPLER, 0, 
-                      slice_param->slice_data_offset + (slice_param->macroblock_offset >> 3));
-            OUT_BATCH(ctx, 
-                      ((slice_param->slice_horizontal_position << 24) |     
-                       (slice_param->slice_vertical_position << 16) |
-                       (127 << 8) | 
-                       (slice_param->macroblock_offset & 0x7)));
-            OUT_BATCH(ctx, slice_param->quantiser_scale_code << 24);
-            ADVANCE_BATCH(ctx);          
-            slice_param++;
-        }
+        BEGIN_BATCH(ctx, 6);
+        OUT_BATCH(ctx, CMD_MEDIA_OBJECT | 4);
+        OUT_BATCH(ctx, 0);
+        OUT_BATCH(ctx, slice_param->slice_data_size - (slice_param->macroblock_offset >> 3));
+        OUT_RELOC(ctx, decode_state->slice_data->bo, 
+                  I915_GEM_DOMAIN_SAMPLER, 0, 
+                  slice_param->slice_data_offset + (slice_param->macroblock_offset >> 3));
+        OUT_BATCH(ctx, 
+                  ((slice_param->slice_horizontal_position << 24) |     
+                   (slice_param->slice_vertical_position << 16) |
+                   (127 << 8) | 
+                   (slice_param->macroblock_offset & 0x7)));
+        OUT_BATCH(ctx, slice_param->quantiser_scale_code << 24);
+        ADVANCE_BATCH(ctx);          
+        slice_param++;
     }
 }
 
@@ -889,7 +886,6 @@ i965_media_mpeg2_decode_init(VADriverContextP ctx)
     dri_bo *bo;
 
     media_state->extended_state.enabled = 1;
-    media_state->indirect_object.bo = NULL;
     dri_bo_unreference(media_state->extended_state.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr, 
                       "vld state", 
@@ -911,7 +907,7 @@ i965_media_mpeg2_decode_init(VADriverContextP ctx)
            media_state->urb.num_cs_entries * media_state->urb.size_cs_entry <= URB_SIZE((&i965->intel)));
 
     /* hook functions */
-    media_state->media_states_setup = i965_media_mpeg2_states_setup;
+    media_state->states_setup = i965_media_mpeg2_states_setup;
     media_state->media_objects = i965_media_mpeg2_objects;
 
 }
@@ -927,7 +923,7 @@ i965_media_mpeg2_init(VADriverContextP ctx)
                                      sizeof(mpeg2_vld_kernels_gen5[0])));
     assert(NUM_MPEG2_VLD_KERNELS <= MAX_INTERFACE_DESC);
 
-    if (IS_IRONLAKE(i965->intel.device_id))
+    if (IS_IGDNG(i965->intel.device_id))
         mpeg2_vld_kernels = mpeg2_vld_kernels_gen5;
     else
         mpeg2_vld_kernels = mpeg2_vld_kernels_gen4;
