@@ -32,12 +32,11 @@
  */
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 
-#include <va/va_backend.h>
-#include "va/x11/va_dricommon.h"
+#include "va_backend.h"
+#include "va_dricommon.h"
 
 #include "intel_batchbuffer.h"
 #include "intel_driver.h"
@@ -75,7 +74,7 @@ static const unsigned int ps_subpic_kernel_static[][4] =
 #include "shaders/render/exa_wm_write.g4b"
 };
 
-/* On IRONLAKE */
+/* On IGDNG */
 static const unsigned int sf_kernel_static_gen5[][4] = 
 {
 #include "shaders/render/exa_sf.g4b.gen5"
@@ -183,8 +182,8 @@ static struct render_kernel *render_kernels = NULL;
 #define URB_SF_ENTRIES	      1
 #define URB_SF_ENTRY_SIZE     2
 
-#define URB_CS_ENTRIES	      1
-#define URB_CS_ENTRY_SIZE     1
+#define URB_CS_ENTRIES	      0
+#define URB_CS_ENTRY_SIZE     0
 
 static void
 i965_render_vs_unit(VADriverContextP ctx)
@@ -198,7 +197,7 @@ i965_render_vs_unit(VADriverContextP ctx)
     vs_state = render_state->vs.state->virtual;
     memset(vs_state, 0, sizeof(*vs_state));
 
-    if (IS_IRONLAKE(i965->intel.device_id))
+    if (IS_IGDNG(i965->intel.device_id))
         vs_state->thread4.nr_urb_entries = URB_VS_ENTRIES >> 2;
     else
         vs_state->thread4.nr_urb_entries = URB_VS_ENTRIES;
@@ -312,7 +311,7 @@ i965_subpic_render_wm_unit(VADriverContextP ctx)
 
     wm_state->thread1.single_program_flow = 1; /* XXX */
 
-    if (IS_IRONLAKE(i965->intel.device_id))
+    if (IS_IGDNG(i965->intel.device_id))
         wm_state->thread1.binding_table_entry_count = 0; /* hardware requirement */
     else
         wm_state->thread1.binding_table_entry_count = 7;
@@ -329,7 +328,7 @@ i965_subpic_render_wm_unit(VADriverContextP ctx)
     wm_state->wm4.stats_enable = 0;
     wm_state->wm4.sampler_state_pointer = render_state->wm.sampler->offset >> 5; 
 
-    if (IS_IRONLAKE(i965->intel.device_id))
+    if (IS_IGDNG(i965->intel.device_id))
         wm_state->wm4.sampler_count = 0;        /* hardware requirement */
     else
         wm_state->wm4.sampler_count = (render_state->wm.sampler_count + 3) / 4;
@@ -375,7 +374,7 @@ i965_render_wm_unit(VADriverContextP ctx)
 
     wm_state->thread1.single_program_flow = 1; /* XXX */
 
-    if (IS_IRONLAKE(i965->intel.device_id))
+    if (IS_IGDNG(i965->intel.device_id))
         wm_state->thread1.binding_table_entry_count = 0;        /* hardware requirement */
     else
         wm_state->thread1.binding_table_entry_count = 7;
@@ -383,8 +382,8 @@ i965_render_wm_unit(VADriverContextP ctx)
     wm_state->thread2.scratch_space_base_pointer = 0;
     wm_state->thread2.per_thread_scratch_space = 0; /* 1024 bytes */
 
-    wm_state->thread3.dispatch_grf_start_reg = 2; /* XXX */
-    wm_state->thread3.const_urb_entry_read_length = 1;
+    wm_state->thread3.dispatch_grf_start_reg = 3; /* XXX */
+    wm_state->thread3.const_urb_entry_read_length = 0;
     wm_state->thread3.const_urb_entry_read_offset = 0;
     wm_state->thread3.urb_entry_read_length = 1; /* XXX */
     wm_state->thread3.urb_entry_read_offset = 0; /* XXX */
@@ -392,7 +391,7 @@ i965_render_wm_unit(VADriverContextP ctx)
     wm_state->wm4.stats_enable = 0;
     wm_state->wm4.sampler_state_pointer = render_state->wm.sampler->offset >> 5; 
 
-    if (IS_IRONLAKE(i965->intel.device_id))
+    if (IS_IGDNG(i965->intel.device_id))
         wm_state->wm4.sampler_count = 0;        /* hardware requirement */
     else 
         wm_state->wm4.sampler_count = (render_state->wm.sampler_count + 3) / 4;
@@ -532,8 +531,7 @@ i965_render_src_surface_state(VADriverContextP ctx,
                               int index,
                               dri_bo *region,
                               unsigned long offset,
-                              int w, int h,
-                              int pitch, int format)
+                              int w, int h)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);  
     struct i965_render_state *render_state = &i965->render_state;
@@ -549,7 +547,7 @@ i965_render_src_surface_state(VADriverContextP ctx,
     ss = ss_bo->virtual;
     memset(ss, 0, sizeof(*ss));
     ss->ss0.surface_type = I965_SURFACE_2D;
-    ss->ss0.surface_format = format;
+    ss->ss0.surface_format = I965_SURFACEFORMAT_R8_UNORM;
     ss->ss0.writedisable_alpha = 0;
     ss->ss0.writedisable_red = 0;
     ss->ss0.writedisable_green = 0;
@@ -567,7 +565,7 @@ i965_render_src_surface_state(VADriverContextP ctx,
     ss->ss2.mip_count = 0;
     ss->ss2.render_target_rotation = 0;
 
-    ss->ss3.pitch = pitch - 1;
+    ss->ss3.pitch = w - 1;
 
     dri_bo_emit_reloc(ss_bo,
                       I915_GEM_DOMAIN_SAMPLER, 0,
@@ -588,7 +586,7 @@ i965_subpic_render_src_surface_state(VADriverContextP ctx,
                               int index,
                               dri_bo *region,
                               unsigned long offset,
-                              int w, int h, int p, int format)
+                              int w, int h, int format)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);  
     struct i965_render_state *render_state = &i965->render_state;
@@ -622,7 +620,7 @@ i965_subpic_render_src_surface_state(VADriverContextP ctx,
     ss->ss2.mip_count = 0;
     ss->ss2.render_target_rotation = 0;
 
-    ss->ss3.pitch = p - 1;
+    ss->ss3.pitch = w - 1;
 
     dri_bo_emit_reloc(ss_bo,
                       I915_GEM_DOMAIN_SAMPLER, 0,
@@ -643,7 +641,6 @@ i965_render_src_surfaces_state(VADriverContextP ctx,
                               VASurfaceID surface)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);  
-    struct i965_render_state *render_state = &i965->render_state;
     struct object_surface *obj_surface;
     int w, h;
     dri_bo *region;
@@ -655,18 +652,12 @@ i965_render_src_surfaces_state(VADriverContextP ctx,
     h = obj_surface->height;
     region = obj_surface->bo;
 
-    i965_render_src_surface_state(ctx, 1, region, 0, w, h, w, I965_SURFACEFORMAT_R8_UNORM);     /* Y */
-    i965_render_src_surface_state(ctx, 2, region, 0, w, h, w, I965_SURFACEFORMAT_R8_UNORM);
-
-    if (render_state->interleaved_uv) {
-        i965_render_src_surface_state(ctx, 3, region, w * h, w / 2, h / 2, w, I965_SURFACEFORMAT_R8G8_UNORM); /* UV */
-        i965_render_src_surface_state(ctx, 4, region, w * h, w / 2, h / 2, w, I965_SURFACEFORMAT_R8G8_UNORM);
-    } else {
-        i965_render_src_surface_state(ctx, 3, region, w * h, w / 2, h / 2, w / 2, I965_SURFACEFORMAT_R8_UNORM); /* U */
-        i965_render_src_surface_state(ctx, 4, region, w * h, w / 2, h / 2, w / 2, I965_SURFACEFORMAT_R8_UNORM);
-        i965_render_src_surface_state(ctx, 5, region, w * h + w * h / 4, w / 2, h / 2, w / 2, I965_SURFACEFORMAT_R8_UNORM);     /* V */
-        i965_render_src_surface_state(ctx, 6, region, w * h + w * h / 4, w / 2, h / 2, w / 2, I965_SURFACEFORMAT_R8_UNORM);
-    }
+    i965_render_src_surface_state(ctx, 1, region, 0, w, h);     /* Y */
+    i965_render_src_surface_state(ctx, 2, region, 0, w, h);
+    i965_render_src_surface_state(ctx, 3, region, w * h + w * h / 4, w / 2, h / 2);     /* V */
+    i965_render_src_surface_state(ctx, 4, region, w * h + w * h / 4, w / 2, h / 2);
+    i965_render_src_surface_state(ctx, 5, region, w * h, w / 2, h / 2); /* U */
+    i965_render_src_surface_state(ctx, 6, region, w * h, w / 2, h / 2);
 }
 
 static void
@@ -687,8 +678,8 @@ i965_subpic_render_src_surfaces_state(VADriverContextP ctx,
     region = obj_surface->bo;
     subpic_region = obj_image->bo;
     /*subpicture surface*/
-    i965_subpic_render_src_surface_state(ctx, 1, subpic_region, 0, obj_subpic->width, obj_subpic->height, obj_subpic->pitch, obj_subpic->format);     
-    i965_subpic_render_src_surface_state(ctx, 2, subpic_region, 0, obj_subpic->width, obj_subpic->height, obj_subpic->pitch, obj_subpic->format);     
+    i965_subpic_render_src_surface_state(ctx, 1, subpic_region, 0, obj_subpic->width, obj_subpic->height, obj_subpic->format);     
+    i965_subpic_render_src_surface_state(ctx, 2, subpic_region, 0, obj_subpic->width, obj_subpic->height, obj_subpic->format);     
 }
 
 static void
@@ -796,81 +787,13 @@ i965_render_binding_table(VADriverContextP ctx)
     dri_bo_unmap(render_state->wm.binding_table);
 }
 
-static unsigned int
-i965_render_do_upload_vertex(VADriverContextP   ctx,
-                             unsigned int       width,
-                             unsigned int       height,
-                             const VARectangle *src_rect,
-                             const VARectangle *dst_rect,
-                             const VARectangle *cliprects,
-                             unsigned int       num_cliprects)
-{
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_render_state *render_state = &i965->render_state;
-    struct intel_region *dest_region = render_state->draw_region;
-    float *vb, tx1, tx2, ty1, ty2;
-    int x1, x2, y1, y2;
-    unsigned int i, n;
-
-    if (!cliprects || num_cliprects == 0) {
-        cliprects = dst_rect;
-        num_cliprects = 1;
-    }
-
-    dri_bo_map(render_state->vb.vertex_buffer, 1);
-    assert(render_state->vb.vertex_buffer->virtual);
-    vb = render_state->vb.vertex_buffer->virtual;
-
-    for (n = 0, i = 0; i < num_cliprects; i++) {
-        x1 = dest_region->x + cliprects[i].x;
-        y1 = dest_region->y + cliprects[i].y;
-        x2 = x1 + cliprects[i].width;
-        y2 = y1 + cliprects[i].height;
-        x1 = MAX(x1, dst_rect->x);
-        y1 = MAX(y1, dst_rect->y);
-        x2 = MIN(x2, dst_rect->x + dst_rect->width);
-        y2 = MIN(y2, dst_rect->y + dst_rect->height);
-
-        if (x2 <= x1 || y2 <= y1)
-            continue;
-
-        const float sx1 = (float)(x1 - dst_rect->x) / (float)dst_rect->width;
-        const float sy1 = (float)(y1 - dst_rect->y) / (float)dst_rect->height;
-        const float sx2 = (float)(x2 - dst_rect->x) / (float)dst_rect->width;
-        const float sy2 = (float)(y2 - dst_rect->y) / (float)dst_rect->height;
-        tx1 = ((float)src_rect->x + sx1 * (float)src_rect->width) / width;
-        ty1 = ((float)src_rect->y + sy1 * (float)src_rect->height) / height;
-        tx2 = ((float)src_rect->x + sx2 * (float)src_rect->width) / width;
-        ty2 = ((float)src_rect->y + sy2 * (float)src_rect->height) / height;
-
-        vb[n++] = tx2;
-        vb[n++] = ty2;
-        vb[n++] = x2;
-        vb[n++] = y2;
-
-        vb[n++] = tx1;
-        vb[n++] = ty2;
-        vb[n++] = x1;
-        vb[n++] = y2;
-
-        vb[n++] = tx1;
-        vb[n++] = ty1;
-        vb[n++] = x1;
-        vb[n++] = y1;
-    }
-
-    dri_bo_unmap(render_state->vb.vertex_buffer);
-    return n / 12;
-}
-
-static unsigned int
+static void 
 i965_subpic_render_upload_vertex(VADriverContextP ctx,
                                  VASurfaceID surface,
-                                 const VARectangle *output_rect,
-                                 const VARectangle *cliprects,
-                                 unsigned int num_cliprects)
-{
+                                 const VARectangle *output_rect)
+{    
     struct i965_driver_data  *i965         = i965_driver_data(ctx);
+    struct i965_render_state *render_state = &i965->render_state;
     struct object_surface    *obj_surface  = SURFACE(surface);
     struct object_subpic     *obj_subpic   = SUBPIC(obj_surface->subpic);
 
@@ -880,6 +803,8 @@ i965_subpic_render_upload_vertex(VADriverContextP ctx,
     const float ssy = (float)output_rect->height / (float)obj_surface->height;
     const float sx  = psx * ssx;
     const float sy  = psy * ssy;
+    float *vb, tx1, tx2, ty1, ty2, x1, x2, y1, y2;
+    int i = 0;
 
     VARectangle dst_rect;
     dst_rect.x      = output_rect->x + sx * (float)obj_subpic->dst_rect.x;
@@ -887,57 +812,106 @@ i965_subpic_render_upload_vertex(VADriverContextP ctx,
     dst_rect.width  = sx * (float)obj_subpic->dst_rect.width;
     dst_rect.height = sy * (float)obj_subpic->dst_rect.height;
 
-    return i965_render_do_upload_vertex(ctx,
-                                        obj_subpic->width, obj_subpic->height,
-                                        &obj_subpic->src_rect, &dst_rect,
-                                        cliprects, num_cliprects);
+    dri_bo_map(render_state->vb.vertex_buffer, 1);
+    assert(render_state->vb.vertex_buffer->virtual);
+    vb = render_state->vb.vertex_buffer->virtual;
+
+    tx1 = (float)obj_subpic->src_rect.x / (float)obj_subpic->width;
+    ty1 = (float)obj_subpic->src_rect.y / (float)obj_subpic->height;
+    tx2 = (float)(obj_subpic->src_rect.x + obj_subpic->src_rect.width) / (float)obj_subpic->width;
+    ty2 = (float)(obj_subpic->src_rect.y + obj_subpic->src_rect.height) / (float)obj_subpic->height;
+
+    x1 = (float)dst_rect.x;
+    y1 = (float)dst_rect.y;
+    x2 = (float)(dst_rect.x + dst_rect.width);
+    y2 = (float)(dst_rect.y + dst_rect.height);
+
+    vb[i++] = tx2;
+    vb[i++] = ty2;
+    vb[i++] = x2;
+    vb[i++] = y2;
+
+    vb[i++] = tx1;
+    vb[i++] = ty2;
+    vb[i++] = x1;
+    vb[i++] = y2;
+
+    vb[i++] = tx1;
+    vb[i++] = ty1;
+    vb[i++] = x1;
+    vb[i++] = y1;
+    dri_bo_unmap(render_state->vb.vertex_buffer);
 }
 
-static unsigned int 
+static void 
 i965_render_upload_vertex(VADriverContextP ctx,
                           VASurfaceID surface,
-                          const VARectangle *src_rect,
-                          const VARectangle *dst_rect,
-                          const VARectangle *cliprects,
-                          unsigned int num_cliprects)
-{
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct object_surface *obj_surface = SURFACE(surface);
-    assert(obj_surface);
-
-    return i965_render_do_upload_vertex(ctx,
-                                        obj_surface->width, obj_surface->height,
-                                        src_rect,
-                                        dst_rect,
-                                        cliprects, num_cliprects);
-}
-
-static void
-i965_render_upload_constants(VADriverContextP ctx)
+                          short srcx,
+                          short srcy,
+                          unsigned short srcw,
+                          unsigned short srch,
+                          short destx,
+                          short desty,
+                          unsigned short destw,
+                          unsigned short desth)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct i965_render_state *render_state = &i965->render_state;
-    unsigned short *constant_buffer;
+    struct intel_region *dest_region = render_state->draw_region;
+    struct object_surface *obj_surface;
+    float *vb;
 
-    dri_bo_map(render_state->curbe.bo, 1);
-    assert(render_state->curbe.bo->virtual);
-    constant_buffer = render_state->curbe.bo->virtual;
+    float u1, v1, u2, v2;
+    int i, width, height;
+    int box_x1 = dest_region->x + destx;
+    int box_y1 = dest_region->y + desty;
+    int box_x2 = box_x1 + destw;
+    int box_y2 = box_y1 + desth;
 
-    if (render_state->interleaved_uv)
-        *constant_buffer = 1;
-    else
-        *constant_buffer = 0;
+    obj_surface = SURFACE(surface);
+    assert(surface);
+    width = obj_surface->width;
+    height = obj_surface->height;
 
-    dri_bo_unmap(render_state->curbe.bo);
+    u1 = (float)srcx / width;
+    v1 = (float)srcy / height;
+    u2 = (float)(srcx + srcw) / width;
+    v2 = (float)(srcy + srch) / height;
+
+    dri_bo_map(render_state->vb.vertex_buffer, 1);
+    assert(render_state->vb.vertex_buffer->virtual);
+    vb = render_state->vb.vertex_buffer->virtual;
+
+    i = 0;
+    vb[i++] = u2;
+    vb[i++] = v2;
+    vb[i++] = (float)box_x2;
+    vb[i++] = (float)box_y2;
+    
+    vb[i++] = u1;
+    vb[i++] = v2;
+    vb[i++] = (float)box_x1;
+    vb[i++] = (float)box_y2;
+
+    vb[i++] = u1;
+    vb[i++] = v1;
+    vb[i++] = (float)box_x1;
+    vb[i++] = (float)box_y1;
+
+    dri_bo_unmap(render_state->vb.vertex_buffer);
 }
 
-static unsigned int
+static void
 i965_surface_render_state_setup(VADriverContextP ctx,
-                                VASurfaceID surface,
-                                const VARectangle *src_rect,
-                                const VARectangle *dst_rect,
-                                const VARectangle *cliprects,
-                                unsigned int num_cliprects)
+                        VASurfaceID surface,
+                        short srcx,
+                        short srcy,
+                        unsigned short srcw,
+                        unsigned short srch,
+                        short destx,
+                        short desty,
+                        unsigned short destw,
+                        unsigned short desth)
 {
     i965_render_vs_unit(ctx);
     i965_render_sf_unit(ctx);
@@ -948,18 +922,21 @@ i965_surface_render_state_setup(VADriverContextP ctx,
     i965_render_cc_viewport(ctx);
     i965_render_cc_unit(ctx);
     i965_render_binding_table(ctx);
-    i965_render_upload_constants(ctx);
-    return i965_render_upload_vertex(ctx, surface, src_rect, dst_rect,
-                                     cliprects, num_cliprects);
+    i965_render_upload_vertex(ctx, surface,
+                              srcx, srcy, srcw, srch,
+                              destx, desty, destw, desth);
 }
-
-static unsigned int
+static void
 i965_subpic_render_state_setup(VADriverContextP ctx,
-                               VASurfaceID surface,
-                               const VARectangle *src_rect,
-                               const VARectangle *dst_rect,
-                               const VARectangle *cliprects,
-                               unsigned int num_cliprects)
+                        VASurfaceID surface,
+                        short srcx,
+                        short srcy,
+                        unsigned short srcw,
+                        unsigned short srch,
+                        short destx,
+                        short desty,
+                        unsigned short destw,
+                        unsigned short desth)
 {
     i965_render_vs_unit(ctx);
     i965_render_sf_unit(ctx);
@@ -970,10 +947,16 @@ i965_subpic_render_state_setup(VADriverContextP ctx,
     i965_render_cc_viewport(ctx);
     i965_subpic_render_cc_unit(ctx);
     i965_render_binding_table(ctx);
-    return i965_subpic_render_upload_vertex(ctx, surface, dst_rect,
-                                            cliprects, num_cliprects);
+
+    VARectangle output_rect;
+    output_rect.x      = destx;
+    output_rect.y      = desty;
+    output_rect.width  = destw;
+    output_rect.height = desth;
+    i965_subpic_render_upload_vertex(ctx, surface, &output_rect);
 }
 
+
 static void
 i965_render_pipeline_select(VADriverContextP ctx)
 {
@@ -996,7 +979,7 @@ i965_render_state_base_address(VADriverContextP ctx)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
 
-    if (IS_IRONLAKE(i965->intel.device_id)) {
+    if (IS_IGDNG(i965->intel.device_id)) {
         BEGIN_BATCH(ctx, 8);
         OUT_BATCH(ctx, CMD_STATE_BASE_ADDRESS | 6);
         OUT_BATCH(ctx, 0 | BASE_ADDRESS_MODIFY);
@@ -1114,20 +1097,6 @@ i965_render_cs_urb_layout(VADriverContextP ctx)
     ADVANCE_BATCH(ctx);
 }
 
-static void
-i965_render_constant_buffer(VADriverContextP ctx)
-{
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_render_state *render_state = &i965->render_state;
-
-    BEGIN_BATCH(ctx, 2);
-    OUT_BATCH(ctx, CMD_CONSTANT_BUFFER | (1 << 8) | (2 - 2));
-    OUT_RELOC(ctx, render_state->curbe.bo,
-              I915_GEM_DOMAIN_INSTRUCTION, 0,
-              URB_CS_ENTRY_SIZE - 1);
-    ADVANCE_BATCH(ctx);    
-}
-
 static void
 i965_render_drawing_rectangle(VADriverContextP ctx)
 {
@@ -1148,7 +1117,7 @@ i965_render_vertex_elements(VADriverContextP ctx)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);  
 
-    if (IS_IRONLAKE(i965->intel.device_id)) {
+    if (IS_IGDNG(i965->intel.device_id)) {
         BEGIN_BATCH(ctx, 5);
         OUT_BATCH(ctx, CMD_VERTEX_ELEMENTS | 3);
         /* offset 0: X,Y -> {X, Y, 1.0, 1.0} */
@@ -1223,7 +1192,7 @@ i965_render_upload_image_palette(
 }
 
 static void
-i965_render_startup(VADriverContextP ctx, unsigned int vb_offset)
+i965_render_startup(VADriverContextP ctx)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct i965_render_state *render_state = &i965->render_state;
@@ -1234,9 +1203,9 @@ i965_render_startup(VADriverContextP ctx, unsigned int vb_offset)
               (0 << VB0_BUFFER_INDEX_SHIFT) |
               VB0_VERTEXDATA |
               ((4 * 4) << VB0_BUFFER_PITCH_SHIFT));
-    OUT_RELOC(ctx, render_state->vb.vertex_buffer, I915_GEM_DOMAIN_VERTEX, 0, vb_offset);
+    OUT_RELOC(ctx, render_state->vb.vertex_buffer, I915_GEM_DOMAIN_VERTEX, 0, 0);
 
-    if (IS_IRONLAKE(i965->intel.device_id))
+    if (IS_IGDNG(i965->intel.device_id))
         OUT_RELOC(ctx, render_state->vb.vertex_buffer, I915_GEM_DOMAIN_VERTEX, 0, 12 * 4);
     else
         OUT_BATCH(ctx, 3);
@@ -1299,10 +1268,8 @@ i965_clear_dest_region(VADriverContextP ctx)
 }
 
 static void
-i965_surface_render_pipeline_setup(VADriverContextP ctx, unsigned int n_rects)
+i965_surface_render_pipeline_setup(VADriverContextP ctx)
 {
-    unsigned int i;
-
     intel_batchbuffer_start_atomic(ctx, 0x1000);
     intel_batchbuffer_emit_mi_flush(ctx);
     i965_clear_dest_region(ctx);
@@ -1314,19 +1281,15 @@ i965_surface_render_pipeline_setup(VADriverContextP ctx, unsigned int n_rects)
     i965_render_pipelined_pointers(ctx);
     i965_render_urb_layout(ctx);
     i965_render_cs_urb_layout(ctx);
-    i965_render_constant_buffer(ctx);
     i965_render_drawing_rectangle(ctx);
     i965_render_vertex_elements(ctx);
-    for (i = 0; i < n_rects; i++)
-        i965_render_startup(ctx, 48 * i);
+    i965_render_startup(ctx);
     intel_batchbuffer_end_atomic(ctx);
 }
 
 static void
-i965_subpic_render_pipeline_setup(VADriverContextP ctx, unsigned int n_rects)
+i965_subpic_render_pipeline_setup(VADriverContextP ctx)
 {
-    unsigned int i;
-
     intel_batchbuffer_start_atomic(ctx, 0x1000);
     intel_batchbuffer_emit_mi_flush(ctx);
     i965_render_pipeline_select(ctx);
@@ -1339,8 +1302,7 @@ i965_subpic_render_pipeline_setup(VADriverContextP ctx, unsigned int n_rects)
     i965_render_cs_urb_layout(ctx);
     i965_render_drawing_rectangle(ctx);
     i965_render_vertex_elements(ctx);
-    for (i = 0; i < n_rects; i++)
-        i965_render_startup(ctx, 48 * i);
+    i965_render_startup(ctx);
     intel_batchbuffer_end_atomic(ctx);
 }
 
@@ -1434,39 +1396,45 @@ i965_render_initialize(VADriverContextP ctx)
 void
 i965_render_put_surface(VADriverContextP ctx,
                         VASurfaceID surface,
-                        const VARectangle *src_rect,
-                        const VARectangle *dst_rect,
-                        const VARectangle *cliprects,
-                        unsigned int num_cliprects)
+                        short srcx,
+                        short srcy,
+                        unsigned short srcw,
+                        unsigned short srch,
+                        short destx,
+                        short desty,
+                        unsigned short destw,
+                        unsigned short desth)
 {
-    unsigned int n_rects;
-
     i965_render_initialize(ctx);
-    n_rects = i965_surface_render_state_setup(ctx, surface,
-                                              src_rect, dst_rect,
-                                              cliprects, num_cliprects);
-    i965_surface_render_pipeline_setup(ctx, n_rects);
+    i965_surface_render_state_setup(ctx, surface,
+                            srcx, srcy, srcw, srch,
+                            destx, desty, destw, desth);
+    i965_surface_render_pipeline_setup(ctx);
     intel_batchbuffer_flush(ctx);
 }
 
 void
 i965_render_put_subpic(VADriverContextP ctx,
-                       VASurfaceID surface,
-                       const VARectangle *src_rect,
-                       const VARectangle *dst_rect,
-                       const VARectangle *cliprects,
-                       unsigned int num_cliprects)
+                        VASurfaceID surface,
+                        short srcx,
+                        short srcy,
+                        unsigned short srcw,
+                        unsigned short srch,
+                        short destx,
+                        short desty,
+                        unsigned short destw,
+                        unsigned short desth)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct object_surface *obj_surface = SURFACE(surface);
     struct object_subpic *obj_subpic = SUBPIC(obj_surface->subpic);
-    unsigned int n_rects;
     assert(obj_subpic);
 
     i965_render_initialize(ctx);
-    n_rects = i965_subpic_render_state_setup(ctx, surface, src_rect, dst_rect,
-                                             cliprects, num_cliprects);
-    i965_subpic_render_pipeline_setup(ctx, n_rects);
+    i965_subpic_render_state_setup(ctx, surface,
+	    srcx, srcy, srcw, srch,
+	    destx, desty, destw, desth);
+    i965_subpic_render_pipeline_setup(ctx);
     i965_render_upload_image_palette(ctx, obj_subpic->image, 0xff);
     intel_batchbuffer_flush(ctx);
 }
@@ -1476,14 +1444,13 @@ Bool
 i965_render_init(VADriverContextP ctx)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct i965_render_state *render_state = &i965->render_state;
     int i;
 
     /* kernel */
     assert(NUM_RENDER_KERNEL == (sizeof(render_kernels_gen5) / 
                                  sizeof(render_kernels_gen5[0])));
 
-    if (IS_IRONLAKE(i965->intel.device_id))
+    if (IS_IGDNG(i965->intel.device_id))
         render_kernels = render_kernels_gen5;
     else
         render_kernels = render_kernels_gen4;
@@ -1497,12 +1464,6 @@ i965_render_init(VADriverContextP ctx)
         dri_bo_subdata(kernel->bo, 0, kernel->size, kernel->bin);
     }
 
-    /* constant buffer */
-    render_state->curbe.bo = dri_bo_alloc(i965->intel.bufmgr,
-                      "constant buffer",
-                      4096, 64);
-    assert(render_state->curbe.bo);
-
     return True;
 }
 
@@ -1513,9 +1474,6 @@ i965_render_terminate(VADriverContextP ctx)
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct i965_render_state *render_state = &i965->render_state;
 
-    dri_bo_unreference(render_state->curbe.bo);
-    render_state->curbe.bo = NULL;
-
     for (i = 0; i < NUM_RENDER_KERNEL; i++) {
         struct render_kernel *kernel = &render_kernels[i];
         
